#! /bin/sh

#  Please report errors and comments to
#  Farid A. Parpia     27 January 2004

MPICH=mpich-1.2.5.2

ANSWER="NO"
until [ "$ANSWER" = "YES" ]; do
  echo -n "Adressing mode (32-bit or 64-bit)? "
  read MODE
  MODE=`echo $MODE | tr '[A-Z]' '[a-z]'`
  if [ "$MODE" = "32" ] || [ "$MODE" = "64" ] || [ "$MODE" = "32-bit" ] || [ "$MODE" = "64-bit" ]; then
    MODE=`echo $MODE | awk -F- '{ print $1 }'`
    ANSWER="YES"
  fi
done

ANSWER="NO"
until [ "$ANSWER" = "YES" ]; do
  echo    "Enter the complete pathname of the directory"
  echo -n " into which MPICH is to be installed: "
  read INSTALL_DIR
  FIRST_CHAR=`echo $INSTALL_DIR | sed 's/^\(.\).*/\1/'`
  if [ "$FIRST_CHAR" = "/" ]; then
    if [ -d $INSTALL_DIR ]; then
      ANS="N"
      until [ "$ANS" = "y" ]; do
        echo -n "Directory $INSTALL_DIR already exists; proceed? (y/n) "
        read ANS
        ANS=`echo $ANS | tr '[A-Z]' '[a-z]'`
        if [ "$ANS" = "y" ] || [ "$ANS" = "yes" ]; then
          ANSWER="YES"
        fi
        if [ "$ANS" = "n" ] || [ "$ANS" = "no" ]; then
          exit 1
        fi
      done
    else
      ANSWER="YES"
    fi
  fi
done

echo -n "Deleting previous mpich source tree ... "
/bin/rm -rf $MPICH
echo "done"

echo -n "Unpacking archive ... "
gunzip -c mpich.tar.gz | tar -xf -
echo "done"

cd $MPICH

#  These are workarounds for:
#    (1) a defective Linux header file
#    (2) an improperly configured C++ compiler

export CFLAGS=-I`pwd`/tmp
mkdir -p ./tmp/sys
cat /usr/include/sys/wait.h | sed 's/(\*(int \*) \&(status))/((int \*) \&(status))/' > ./tmp/sys/wait.h
if [ "$MODE" = "64" ]; then
  ADDLCPPFLAGS=-I/opt/cross/include/backward
fi

#  Return to normal processing

echo -n "Configuring ... "
export CC="xlc_r -q$MODE"
export FC="xlf_r -q$MODE"
export F90="xlf90_r -q$MODE"
./configure -c++="xlC_r -q$MODE $ADDLCPPFLAGS" \
            -opt="-O -qmaxmem=-1 -qarch=auto -qtune=auto" \
            -prefix=$INSTALL_DIR \
            --with-device=ch_p4 \
            --with-comm=shared \
            > configure.log.$$ 2>&1
echo "done; please examine file $MPICH/configure.log.$$"

echo -n "Compiling ... "
make > make.log.$$ 2>&1
echo "done; please examine file $MPICH/make.log.$$"

echo -n "Installing ... "
make install > make.install.log.$$ 2>&1
echo "done; please examine file $MPICH/make.install.log.$$"

cd ..
SETUP=setup.$$
echo "#! /bin/sh" > $SETUP
echo "" >> $SETUP
echo "export PATH=$INSTALL_DIR/bin:\$PATH" >> $SETUP
echo "export LIBPATH=$INSTALL_DIR/lib:\$COMPILER_ROOT/lib:\$LIBPATH" >> $SETUP
chmod u+x $SETUP

echo "Korn or Bourne shell users may issue the command"
echo " . ./$SETUP"
echo "to modify PATH and LIBPATH settings."

echo "Please be aware that an ungraceful termination"
echo " of MPICH will cause shared-memory segments and"
echo " semaphores to be abandoned; use"
echo "   ipcs"
echo " to determine the ID of each such object, and"
echo "   ipcrm -m ID"
echo " or"
echo "   ipcrm -s ID"
echo " to remove them."

