/* IBM_PROLOG_BEGIN_TAG                                                   */
/* This is an automatically generated prolog.                             */
/*                                                                        */
/*                                                                        */
/*                                                                        */
/* Licensed Materials - Property of IBM                                   */
/*                                                                        */
/* Restricted Materials of IBM                                            */
/*                                                                        */
/* (C) COPYRIGHT International Business Machines Corp. 1997,2001          */
/* All Rights Reserved                                                    */
/*                                                                        */
/* US Government Users Restricted Rights - Use, duplication or            */
/* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      */
/*                                                                        */
/* IBM_PROLOG_END_TAG                                                     */
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <assert.h>
#include <memory.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>

#include "tec_eeif.h"

typedef enum
{
	DIGIT,
	STRING
} event_attr_types_t;

/* Global Variables */
#define MAX_STR_LEN	2048
#define MAX_CACHE_FLUSHING	480000 

char tecEvent[MAX_STR_LEN];
char *theMessage = "UNKNOWN", *theAdmin = "UNKNOWN", *theSeverity = "UNKNOWN";



void
PrintUsage(char **argv)
{
    fprintf(stderr, "Usage: tecad_pssp [-l cfgfile] [-Cc] [-m message] [-a tiv_admin_name] [-s severity] [-p port]\n");
    exit(3);
}


char *
strCopy(char *dest, const char *src, size_t n)
{
	char *str;
	int size;

	if (src == NULL) return(NULL);

	size = strlen(src)+1;
	if (size >= n) size = n;

	if (dest == NULL) {
		if ((dest = (char *)calloc(size, sizeof(char))) == NULL) {
			assert(dest);
		}
	}

	str = strncpy(dest, src, size);
	str[size-1] = '\0';

	return(str);
}


char *
strCat(char *dest, const char *src, size_t n)
{
	char *str;
	int size;

	if (src == NULL) return(NULL);
	if (dest == NULL) return(NULL);

	size = strlen(src)+ strlen(dest) + 1;
	if (size >= n) size = n;

	strncat(dest, src, size);
	dest[size-1] = '\0';

	return(dest);
}

char *
ValidateSeverity(char *sev)
{
	char *severity;

	severity = sev;
	if( strcmp(sev,"FATAL") &&
		strcmp(sev,"CRITICAL") &&
		strcmp(sev,"WARNING") &&
		strcmp(sev,"MINOR") &&
		strcmp(sev,"HARMLESS")) 
		severity = strCopy(NULL,"UNKNOWN", MAX_STR_LEN);

	return(severity);
}

void 
getEventAttr(char *name, char *value, int type) {
	if (name == NULL) {
		return;
	} else {
		strCat(tecEvent,name, MAX_STR_LEN);

		strCat(tecEvent,"=", MAX_STR_LEN);

		if (type == STRING) {
			strCat(tecEvent,"'", MAX_STR_LEN);
		}

		if (value != NULL) {
			strCat(tecEvent,value, MAX_STR_LEN);
		}

		if (type == STRING) {
			strCat(tecEvent,"'", MAX_STR_LEN);
		}

		strCat(tecEvent, ";", MAX_STR_LEN);
	}
}

void
getTecEvent(void) {
	char *temp;
	char tmp_str[256];
	char *p, *tc, *tms;
	int i;

	FILE *cmdfp;
	time_t ts;

	tecEvent[0] = '\0';

	/* get class name */
	temp = strCopy(NULL, getenv("ERRM_RSRC_CLASS_PNAME"),MAX_STR_LEN);
	while ((p=strstr(temp, ".")) != NULL) {
		*p = '_';
	}
	strCopy(tecEvent, "CSM_ERRM_", MAX_STR_LEN);
	strCat(tecEvent, temp, MAX_STR_LEN);
	free(temp);
	strCat(tecEvent, ";", MAX_STR_LEN);

	getEventAttr("source", "CSM", STRING);
	getEventAttr("sub_source", "ERRM", STRING);

	/* get hostname and event origin*/
	sprintf(tmp_str,"host %s", getenv("ERRM_NODE_NAME"));
	if ((cmdfp = popen(tmp_str,"r")) == NULL)
	{
		fprintf(stderr,"The hostname command failed\n");
		exit(4);
	}
	fscanf(cmdfp,"%s",tmp_str); /* get the hostname */
	getEventAttr("hostname", tmp_str, STRING);

	while (fscanf(cmdfp,"%s",tmp_str)) {
		if (isdigit(*tmp_str)) {
			getEventAttr("origin", tmp_str, STRING);
			break;
		}
	}
	pclose(cmdfp);


	/* get adapter_host and sub_origin*/
	sprintf(tmp_str,"host `hostname`");
	if ((cmdfp = popen(tmp_str,"r")) == NULL)
	{
		fprintf(stderr,"The hostname command failed\n");
		exit(5);
	}
	fscanf(cmdfp,"%s",tmp_str); /* get the hostname */
	getEventAttr("adapter_host", tmp_str, STRING);

	while (fscanf(cmdfp,"%s",tmp_str)) {
		if (isdigit(*tmp_str)) {
			getEventAttr("sub_origin", tmp_str, STRING);
			break;
		}
	}
	pclose(cmdfp);

	/* get date */
	temp =  strCopy(NULL, getenv("ERRM_TIME"), MAX_STR_LEN);
	if ((tms = strstr(temp, ",")) != NULL) {
		*tms = '\0';
		tms++;
	}

	ts = (time_t)strtoul(temp, NULL, 10);

	tc = (char *)ctime((const time_t *)(&ts));
	p = tc + 19;
	*p ='\0';
	*(p+5) ='\0';
	p++;
	sprintf(tmp_str, "%s.%s %s", tc, tms, p); 
	getEventAttr("date", tmp_str, STRING);
	free(temp);

	/* get other attribute */
	getEventAttr("administrator", theAdmin, STRING);
	getEventAttr("severity", theSeverity, DIGIT);
	getEventAttr("msg", theMessage, STRING);


	getEventAttr("errm_cond_handle", getenv("ERRM_COND_HANDLE"), STRING);
	getEventAttr("errm_cond_name", getenv("ERRM_COND_NAME"), STRING);
	getEventAttr("errm_cond_severity", getenv("ERRM_COND_SEVERITY"), STRING);
	getEventAttr("errm_cond_severityid", getenv("ERRM_COND_SEVERITYID"), DIGIT);


	getEventAttr("errm_er_handle", getenv("ERRM_ER_HANDLE"), STRING);
	getEventAttr("errm_er_name", getenv("ERRM_ER_NAME"), STRING);


	getEventAttr("errm_rsrc_handle", getenv("ERRM_RSRC_HANDLE"), STRING);
	getEventAttr("errm_rsrc_name", getenv("ERRM_RSRC_NAME"), STRING);
	getEventAttr("errm_rsrc_class_name", getenv("ERRM_RSRC_CLASS_NAME"), 
			STRING);
	getEventAttr("errm_rsrc_class_pname", getenv("ERRM_RSRC_CLASS_PNAME"), 
			STRING);
	getEventAttr("errm_rsrc_type", getenv("ERRM_RSRC_TYPE"), DIGIT);


	getEventAttr("errm_time", getenv("ERRM_TIME"), STRING);
	getEventAttr("errm_type", getenv("ERRM_TYPE"), STRING);
	getEventAttr("errm_typeid", getenv("ERRM_TYPEID"), DIGIT);
	getEventAttr("errm_expr", getenv("ERRM_EXPR"), STRING);
	getEventAttr("errm_attr_name", getenv("ERRM_ATTR_NAME"), STRING);
	getEventAttr("errm_attr_pname", getenv("ERRM_ATTR_PNAME"), STRING);
	getEventAttr("errm_data_type", getenv("ERRM_DATA_TYPE"), STRING);
	getEventAttr("errm_value", getenv("ERRM_VALUE"), STRING);
	getEventAttr("errm_sd_data_type", getenv("ERRM_SD_DATA_TYPE"), STRING);
	getEventAttr("errm_node_name", getenv("ERRM_NODE_NAME"), STRING);
	getEventAttr("errm_node_namelist", getenv("ERRM_NODE_NAMELIST"), STRING);
}


int
main(int argc, char *argv[])
{
	int c;
	int rc;
	extern int     optind;
	extern char    *optarg;
	char           *location = NULL;
	char           cfg_file_name[FILE_LEN];
	char *default_cfg_file = "/opt/csm/tecad/conf/tecaderrm.conf";
	unsigned short port = 0;
	tec_handle_t   th;
	extern int     tec_errno;
	int            connectionless = 0;
	int            oneway = 0;
	tec_delivery   type = use_default;
	struct         stat   sbuf;
	int	debug = 0;




	cfg_file_name[0]= '\0';


	while ((c = getopt(argc, argv, "Ccl:a:m:p:s:d")) != -1) {
		switch (c) {
		case 'l':
			/* Specifies location of the agent configuration file */
			strCopy(cfg_file_name, optarg, FILE_LEN);
			if (stat(cfg_file_name, &sbuf) != 0) {
				/* not existant */
				fprintf(stderr, "%s: Not existant\n", cfg_file_name);
				PrintUsage(argv);
			}
			break;
		case 'p':
			/* Specifies the port to connect to on the Server */
			if (!isdigit(*optarg)) {
				PrintUsage(argv);
			}
			port = atol(optarg);
			break;
		case 'C':
			oneway++;
		case 'c':
			connectionless++;
			type = connection_less;
			break;
		case 'm':
			theMessage = strCopy(NULL, optarg, MAX_STR_LEN);
			break;
		case 's':
			theSeverity = ValidateSeverity(optarg);
			break;
		case 'a':
			theAdmin = strCopy(NULL, optarg, MAX_STR_LEN);
			break;  
		case 'd':
			debug = 1;
			break;
		default:
			PrintUsage(argv);
			break;
		}
	}

	if (optind + 1 == argc) {
		location = argv[optind];
	}

	if( cfg_file_name[0] == '\0')
	{
		strCopy(cfg_file_name, default_cfg_file, FILE_LEN);
		if (stat(cfg_file_name, &sbuf) != 0)
		{
			/* not existant */
			fprintf(stderr, "%s: Not existant\n", cfg_file_name);
			PrintUsage(argv);
		}
	}

	getTecEvent();

	if( debug) printf("%s\n", tecEvent);

	tec_agent_init(cfg_file_name); 

	if ((th = tec_create_handle(location, port, oneway, type)) == NULL) {
		fprintf(stderr, "tec_create_handle failed, errno = %d\n", tec_errno);
		exit(1); 
	}


	/* tec_put_event will send the event to the selected server */
	rc = tec_put_event(th, tecEvent);

	if (rc == -1) {
		fprintf(stderr, "tec_put_event failed, errno = %d\n", tec_errno);
		exit(2);
	}

	/* make sure the event is gone, otherwise wait util the limted times
	 * are reached, the max time to wait is 2 minutes.
	 * */
	c = 1;
	while ((c < MAX_CACHE_FLUSHING) && tec_flushing_cache(th)) { 
		c++;
		ed_sleep(0,250); 
	} 

	if (c == MAX_CACHE_FLUSHING) {
	       fprintf(stderr, "Max number of cache flushing reached, event may still in the cache.\n");
	}
	if (debug) printf("Event cache had been flushed %i time(s)\n", c);

	/* close the connection and exit */
	tec_destroy_handle(th);

	exit(0);
}  
